(function(module) {
    "use strict";

    var constants = Object.freeze({
        'name': "Amazon-S3",
        'admin': {
            'route': '/plugins/s3-uploads',
            'icon': 'fa-picture-o'
        },
        config: {
            bucket: process.env.S3_UPLOADS_BUCKET || "",
            accessKeyId: process.env.AWS_ACCESS_KEY_ID || "",
            secretAccessKey: process.env.AWS_SECRET_ACCESS_KEY || "",
            host: process.env.S3_UPLOADS_HOST || "",
            path: process.env.S3_UPLOADS_PATH || ""
        }
    });

    var Package = require("./package.json"),
        AWS = require('aws-sdk'),
        mime = require("mime"),
        uuid = require("uuid").v4,
        fs = require('fs'),
        path = require('path'),
        winston = module.parent.require('winston'),
        s3bucket = new AWS.S3({ params: { Bucket: constants.config.bucket || "" } }),
        app;


    var AmazonS3 = {};

    AmazonS3.load = function(params, callback) {
        function render(req, res, next) {
            res.render('admin/plugins/s3-uploads', constants.config);
        }

        params.router.get('/admin/plugins/s3-uploads', params.middleware.admin.buildHeader, render);
        params.router.get('/api/admin/plugins/s3-uploads', render);
        winston.info('[plugins/' + Package.name + '] Settings loaded');

        callback();
    };

    AmazonS3.uploadImage = function (data, callback) {
        winston.log('info', '[plugins/%s] %s', Package.name, 'starting upload image...', data);

        if (!constants.config.bucket || !constants.config.accessKeyId || !constants.config.secretAccessKey) {
            return callback(new Error('AWS_ACCESS_KEY_ID or AWS_SECRET_ACCESS_KEY or S3_UPLOADS_BUCKET'));
        }

        var image = data.image;

        if (!image) {
            return callback(new Error('invalid image'));
        }

        var type = image.url ? 'url' : 'file';

        if (type === 'file' && !image.path) {
            return callback(new Error('invalid image path'));
        }

        var imageData = type === 'file' ? fs.createReadStream(image.path) : image.url;
        var imageName = type === 'file' ? image.path : image.url;
        uploadToAmazon(imageData, path.extname(imageName), callback);
    };

    AmazonS3.uploadFile = function (data, callback) {
        winston.info('[plugins/' + Package.name + '] starting upload file...');

        if (!constants.config.bucket || !constants.config.accessKeyId || !constants.config.secretAccessKey) {
            return callback(new Error('AWS_ACCESS_KEY_ID or AWS_SECRET_ACCESS_KEY or S3_UPLOADS_BUCKET'));
        }

        var file = data.file;

        if (!file) {
            return callback(new Error('invalid file'));
        }

        if (!file.path) {
            return callback(new Error('invalid file path'));
        }

        uploadToAmazon(fs.createReadStream(image.path), path.extname(image.path), callback);
    };

    function uploadToAmazon(buffer, name, callback) {
        var params = {
            ACL: "public-read",
            Key: uuid() + name,
            Body: buffer,
            ContentLength: buffer.length,
            ContentType: mime.lookup(image.name)
        };

        s3bucket.putObject(params, function (err) {
            if (err) {
                return callback(err);
            }

            callback(null, {
                name: name,
                // Use protocol-less urls so that both HTTP and HTTPS work:
                url: "//" + constants.config.bucket + ".s3.amazonaws.com/" + params.Key
            });
        });
    }


    AmazonS3.admin = {
        menu: function(custom_header, callback) {
            custom_header.plugins.push({
                "route": constants.admin.route,
                "icon": constants.admin.icon,
                "name": constants.name
            });

            callback(null, custom_header);
        }
    };

    module.exports = AmazonS3;
}(module));
